/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_VISUALIZATION_VISUALIZATIONSOCB_SKELETON_H
#define JAC_AP_VISUALIZATION_VISUALIZATIONSOCB_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/visualization/visualizationsocb_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace visualization {
namespace skeleton {
namespace events
{
    using CameraFEH265ImagePDataViz = ara::com::internal::skeleton::event::EventAdapter<::CameraH265Image>;
    using LidarInno3PPC2Viz = ara::com::internal::skeleton::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    using LidarInno4PPC2Viz = ara::com::internal::skeleton::event::EventAdapter<::perception::LiDAR2LiDARPerception>;
    static constexpr ara::com::internal::EntityId VisualizationSoCBCameraFEH265ImagePDataVizId = 7559U; //CameraFEH265ImagePDataViz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationSoCBLidarInno3PPC2VizId = 59449U; //LidarInno3PPC2Viz_event_hash
    static constexpr ara::com::internal::EntityId VisualizationSoCBLidarInno4PPC2VizId = 16882U; //LidarInno4PPC2Viz_event_hash
}

namespace methods
{
}

namespace fields
{
}

class VisualizationSoCBSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultCameraFEH265ImagePDataViz = CameraFEH265ImagePDataViz.Initialize();
        ThrowError(resultCameraFEH265ImagePDataViz);
        const ara::core::Result<void> resultLidarInno3PPC2Viz = LidarInno3PPC2Viz.Initialize();
        ThrowError(resultLidarInno3PPC2Viz);
        const ara::core::Result<void> resultLidarInno4PPC2Viz = LidarInno4PPC2Viz.Initialize();
        ThrowError(resultLidarInno4PPC2Viz);
    }

    VisualizationSoCBSkeleton& operator=(const VisualizationSoCBSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceId, mode)),
              CameraFEH265ImagePDataViz(ptr->GetSkeleton(), events::VisualizationSoCBCameraFEH265ImagePDataVizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceId),
              LidarInno3PPC2Viz(ptr->GetSkeleton(), events::VisualizationSoCBLidarInno3PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceId),
              LidarInno4PPC2Viz(ptr->GetSkeleton(), events::VisualizationSoCBLidarInno4PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceSpec, mode)),
              CameraFEH265ImagePDataViz(ptr->GetSkeleton(), events::VisualizationSoCBCameraFEH265ImagePDataVizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceSpec),
              LidarInno3PPC2Viz(ptr->GetSkeleton(), events::VisualizationSoCBLidarInno3PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceSpec),
              LidarInno4PPC2Viz(ptr->GetSkeleton(), events::VisualizationSoCBLidarInno4PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceContainer, mode)),
              CameraFEH265ImagePDataViz(ptr->GetSkeleton(), events::VisualizationSoCBCameraFEH265ImagePDataVizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceContainer),
              LidarInno3PPC2Viz(ptr->GetSkeleton(), events::VisualizationSoCBLidarInno3PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceContainer),
              LidarInno4PPC2Viz(ptr->GetSkeleton(), events::VisualizationSoCBLidarInno4PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              CameraFEH265ImagePDataViz(std::move(other.CameraFEH265ImagePDataViz)),
              LidarInno3PPC2Viz(std::move(other.LidarInno3PPC2Viz)),
              LidarInno4PPC2Viz(std::move(other.LidarInno4PPC2Viz)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                CameraFEH265ImagePDataViz = std::move(other.CameraFEH265ImagePDataViz);
                LidarInno3PPC2Viz = std::move(other.LidarInno3PPC2Viz);
                LidarInno4PPC2Viz = std::move(other.LidarInno4PPC2Viz);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::CameraFEH265ImagePDataViz GetCameraFEH265ImagePDataViz()
        {
            return std::move(CameraFEH265ImagePDataViz);
        }
        events::LidarInno3PPC2Viz GetLidarInno3PPC2Viz()
        {
            return std::move(LidarInno3PPC2Viz);
        }
        events::LidarInno4PPC2Viz GetLidarInno4PPC2Viz()
        {
            return std::move(LidarInno4PPC2Viz);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = CameraFEH265ImagePDataViz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LidarInno3PPC2Viz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LidarInno4PPC2Viz.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::CameraFEH265ImagePDataViz CameraFEH265ImagePDataViz;
        events::LidarInno3PPC2Viz LidarInno3PPC2Viz;
        events::LidarInno4PPC2Viz LidarInno4PPC2Viz;
    };
    explicit VisualizationSoCBSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceId, mode)),
          CameraFEH265ImagePDataViz(skeletonAdapter->GetSkeleton(), events::VisualizationSoCBCameraFEH265ImagePDataVizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceId),
          LidarInno3PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationSoCBLidarInno3PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceId),
          LidarInno4PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationSoCBLidarInno4PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit VisualizationSoCBSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceSpec, mode)),
          CameraFEH265ImagePDataViz(skeletonAdapter->GetSkeleton(), events::VisualizationSoCBCameraFEH265ImagePDataVizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceSpec),
          LidarInno3PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationSoCBLidarInno3PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceSpec),
          LidarInno4PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationSoCBLidarInno4PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit VisualizationSoCBSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceContainer, mode)),
          CameraFEH265ImagePDataViz(skeletonAdapter->GetSkeleton(), events::VisualizationSoCBCameraFEH265ImagePDataVizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceContainer),
          LidarInno3PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationSoCBLidarInno3PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceContainer),
          LidarInno4PPC2Viz(skeletonAdapter->GetSkeleton(), events::VisualizationSoCBLidarInno4PPC2VizId, ::jac::ap::visualization::VisualizationSoCB::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    VisualizationSoCBSkeleton(const VisualizationSoCBSkeleton&) = delete;

    VisualizationSoCBSkeleton(VisualizationSoCBSkeleton&&) = default;
    VisualizationSoCBSkeleton& operator=(VisualizationSoCBSkeleton&&) = default;
    VisualizationSoCBSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          CameraFEH265ImagePDataViz(token.GetCameraFEH265ImagePDataViz()),
          LidarInno3PPC2Viz(token.GetLidarInno3PPC2Viz()),
          LidarInno4PPC2Viz(token.GetLidarInno4PPC2Viz()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~VisualizationSoCBSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&VisualizationSoCBSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::CameraFEH265ImagePDataViz CameraFEH265ImagePDataViz;
    events::LidarInno3PPC2Viz LidarInno3PPC2Viz;
    events::LidarInno4PPC2Viz LidarInno4PPC2Viz;
};
} // namespace skeleton
} // namespace visualization
} // namespace ap
} // namespace jac

#endif // JAC_AP_VISUALIZATION_VISUALIZATIONSOCB_SKELETON_H
